'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }();

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

/*
 * Copyright (C) 2013 Google Inc. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *     * Redistributions of source code must retain the above copyright
 * notice, this list of conditions and the following disclaimer.
 *     * Redistributions in binary form must reproduce the above
 * copyright notice, this list of conditions and the following disclaimer
 * in the documentation and/or other materials provided with the
 * distribution.
 *     * Neither the name of Google Inc. nor the names of its
 * contributors may be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS
 * "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT
 * LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR
 * A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT
 * OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE,
 * DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY
 * THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
 * (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE
 * OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

var TextUtils = exports.TextUtils = {
  /**
   * @param {string} char
   * @return {boolean}
   */
  isStopChar: function isStopChar(char) {
    return char > ' ' && char < '0' || char > '9' && char < 'A' || char > 'Z' && char < '_' || char > '_' && char < 'a' || char > 'z' && char <= '~';
  },

  /**
   * @param {string} char
   * @return {boolean}
   */
  isWordChar: function isWordChar(char) {
    return !TextUtils.isStopChar(char) && !TextUtils.isSpaceChar(char);
  },

  /**
   * @param {string} char
   * @return {boolean}
   */
  isSpaceChar: function isSpaceChar(char) {
    return TextUtils._SpaceCharRegex.test(char);
  },

  /**
   * @param {string} word
   * @return {boolean}
   */
  isWord: function isWord(word) {
    for (var i = 0; i < word.length; ++i) {
      if (!TextUtils.isWordChar(word.charAt(i))) return false;
    }
    return true;
  },

  /**
   * @param {string} char
   * @return {boolean}
   */
  isOpeningBraceChar: function isOpeningBraceChar(char) {
    return char === '(' || char === '{';
  },

  /**
   * @param {string} char
   * @return {boolean}
   */
  isClosingBraceChar: function isClosingBraceChar(char) {
    return char === ')' || char === '}';
  },

  /**
   * @param {string} char
   * @return {boolean}
   */
  isBraceChar: function isBraceChar(char) {
    return TextUtils.isOpeningBraceChar(char) || TextUtils.isClosingBraceChar(char);
  },

  /**
   * @param {string} text
   * @param {function(string):boolean} isWordChar
   * @param {function(string)} wordCallback
   */
  textToWords: function textToWords(text, isWordChar, wordCallback) {
    var startWord = -1;
    for (var i = 0; i < text.length; ++i) {
      if (!isWordChar(text.charAt(i))) {
        if (startWord !== -1) wordCallback(text.substring(startWord, i));
        startWord = -1;
      } else if (startWord === -1) {
        startWord = i;
      }
    }
    if (startWord !== -1) wordCallback(text.substring(startWord));
  },

  /**
   * @param {string} line
   * @return {string}
   */
  lineIndent: function lineIndent(line) {
    var indentation = 0;
    while (indentation < line.length && TextUtils.isSpaceChar(line.charAt(indentation))) {
      ++indentation;
    }return line.substr(0, indentation);
  },

  /**
   * @param {string} text
   * @return {boolean}
   */
  isUpperCase: function isUpperCase(text) {
    return text === text.toUpperCase();
  },

  /**
   * @param {string} text
   * @return {boolean}
   */
  isLowerCase: function isLowerCase(text) {
    return text === text.toLowerCase();
  },

  /**
   * @param {string} text
   * @param {!Array<!RegExp>} regexes
   * @return {!Array<{value: string, position: number, regexIndex: number}>}
   */
  splitStringByRegexes: function splitStringByRegexes(text, regexes) {
    var matches = [];
    var globalRegexes = [];
    for (var i = 0; i < regexes.length; i++) {
      var regex = regexes[i];
      if (!regex.global) globalRegexes.push(new RegExp(regex.source, regex.flags ? regex.flags + 'g' : 'g'));else globalRegexes.push(regex);
    }
    doSplit(text, 0, 0);
    return matches;

    /**
     * @param {string} text
     * @param {number} regexIndex
     * @param {number} startIndex
     */
    function doSplit(text, regexIndex, startIndex) {
      if (regexIndex >= globalRegexes.length) {
        // Set regexIndex as -1 if text did not match with any regular expression
        matches.push({ value: text, position: startIndex, regexIndex: -1 });
        return;
      }
      var regex = globalRegexes[regexIndex];
      var currentIndex = 0;
      var result;
      regex.lastIndex = 0;
      //prettier-ignore
      while ((result = regex.exec(text)) !== null) {
        // eslint-disable-line no-cond-assign
        var stringBeforeMatch = text.substring(currentIndex, result.index);
        if (stringBeforeMatch) doSplit(stringBeforeMatch, regexIndex + 1, startIndex + currentIndex);
        var match = result[0];
        matches.push({
          value: match,
          position: startIndex + result.index,
          regexIndex: regexIndex
        });
        currentIndex = result.index + match.length;
      }
      var stringAfterMatches = text.substring(currentIndex);
      if (stringAfterMatches) doSplit(stringAfterMatches, regexIndex + 1, startIndex + currentIndex);
    }
  }
};

TextUtils._SpaceCharRegex = /\s/;

/**
 * @enum {string}
 */
TextUtils.Indent = {
  TwoSpaces: '  ',
  FourSpaces: '    ',
  EightSpaces: '        ',
  TabCharacter: '\t'
};

/**
 * @unrestricted
 */
TextUtils.BalancedJSONTokenizer = function () {
  /**
   * @param {function(string)} callback
   * @param {boolean=} findMultiple
   */
  function _class(callback, findMultiple) {
    _classCallCheck(this, _class);

    this._callback = callback;
    this._index = 0;
    this._balance = 0;
    this._buffer = '';
    this._findMultiple = findMultiple || false;
    this._closingDoubleQuoteRegex = /[^\\](?:\\\\)*"/g;
  }

  /**
   * @param {string} chunk
   * @return {boolean}
   */


  _createClass(_class, [{
    key: 'write',
    value: function write(chunk) {
      this._buffer += chunk;
      var lastIndex = this._buffer.length;
      var buffer = this._buffer;
      for (var index = this._index; index < lastIndex; ++index) {
        var character = buffer[index];
        if (character === '"') {
          this._closingDoubleQuoteRegex.lastIndex = index;
          if (!this._closingDoubleQuoteRegex.test(buffer)) break;
          index = this._closingDoubleQuoteRegex.lastIndex - 1;
        } else if (character === '{') {
          ++this._balance;
        } else if (character === '}') {
          --this._balance;
          if (this._balance < 0) {
            this._reportBalanced();
            return false;
          }
          if (!this._balance) {
            this._lastBalancedIndex = index + 1;
            if (!this._findMultiple) break;
          }
        } else if (character === ']' && !this._balance) {
          this._reportBalanced();
          return false;
        }
      }
      this._index = index;
      this._reportBalanced();
      return true;
    }
  }, {
    key: '_reportBalanced',
    value: function _reportBalanced() {
      if (!this._lastBalancedIndex) return;
      this._callback(this._buffer.slice(0, this._lastBalancedIndex));
      this._buffer = this._buffer.slice(this._lastBalancedIndex);
      this._index -= this._lastBalancedIndex;
      this._lastBalancedIndex = 0;
    }

    /**
     * @return {string}
     */

  }, {
    key: 'remainder',
    value: function remainder() {
      return this._buffer;
    }
  }]);

  return _class;
}();

/**
 * @interface
 */
TextUtils.TokenizerFactory = function () {};

TextUtils.TokenizerFactory.prototype = {
  /**
   * @param {string} mimeType
   * @return {function(string, function(string, ?string, number, number))}
   */
  createTokenizer: function createTokenizer() {}
};

/**
 * @param {string} text
 * @return {boolean}
 */
TextUtils.isMinified = function (text) {
  var kMaxNonMinifiedLength = 500;
  var linesToCheck = 10;
  var lastPosition = 0;
  do {
    var eolIndex = text.indexOf('\n', lastPosition);
    if (eolIndex < 0) eolIndex = text.length;
    if (eolIndex - lastPosition > kMaxNonMinifiedLength && text.substr(lastPosition, 3) !== '//#') return true;
    lastPosition = eolIndex + 1;
  } while (--linesToCheck >= 0 && lastPosition < text.length);
  return false;
};