'use strict';

var _createClass = function () { function defineProperties(target, props) { for (var i = 0; i < props.length; i++) { var descriptor = props[i]; descriptor.enumerable = descriptor.enumerable || false; descriptor.configurable = true; if ("value" in descriptor) descriptor.writable = true; Object.defineProperty(target, descriptor.key, descriptor); } } return function (Constructor, protoProps, staticProps) { if (protoProps) defineProperties(Constructor.prototype, protoProps); if (staticProps) defineProperties(Constructor, staticProps); return Constructor; }; }(); /* global CodeMirror */


var _Dictionary = require('./common/Dictionary');

var _Dictionary2 = _interopRequireDefault(_Dictionary);

var _TextUtils = require('./common/TextUtils');

var _Keys = require('./common/Keys');

var _Keys2 = _interopRequireDefault(_Keys);

var _keywords = require('./common/keywords');

var keywords = _interopRequireWildcard(_keywords);

function _interopRequireWildcard(obj) { if (obj && obj.__esModule) { return obj; } else { var newObj = {}; if (obj != null) { for (var key in obj) { if (Object.prototype.hasOwnProperty.call(obj, key)) newObj[key] = obj[key]; } } newObj.default = obj; return newObj; } }

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _classCallCheck(instance, Constructor) { if (!(instance instanceof Constructor)) { throw new TypeError("Cannot call a class as a function"); } }

var CSS = 'css';

var isCSSWordChar = function isCSSWordChar(c) {
  return _TextUtils.TextUtils.isWordChar(c) || c === '-';
};

CodeMirror.defineOption('autocomplete', false, function (cm, value) {
  if (value === false) {
    return;
  }

  if (CodeMirror.autocomplete && CodeMirror.autocomplete.core) {
    CodeMirror.autocomplete.core.disconnect();
  }

  var autoComplete = new Autocomplete(cm);
  CodeMirror.registerHelper('autocomplete', 'core', autoComplete);
});

var Autocomplete = function () {
  function Autocomplete(cm) {
    _classCallCheck(this, Autocomplete);

    _initialiseProps.call(this);

    var hint = document.createElement('span');
    hint.className = 'widget-autocomplete';
    this.hint = hint;
    this.dictionary = new _Dictionary2.default();
    this.mark = null;

    var mode = this.mode = cm.getOption('mode');
    console.log(mode);

    // keywords.css = cm.hintWords.css;

    this.cm = cm;
    cm.on('change', this.change);
    cm.on('keydown', this.keydown);
    cm.on('beforeChange', this.beforeChange);
    cm.on('cursorActivity', this.cursorActivity);
    cm.on('mousedown', this.clear);

    // this.watch = setInterval(() => console.log(this.currentHint), 1000);

    window.remy = this;

    this.reset();
  }

  _createClass(Autocomplete, [{
    key: 'disconnect',
    value: function disconnect() {
      var cm = this.cm;
      // clearInterval(this.watch);
      cm.off('change', this.change);
      cm.off('keydown', this.keydown);
      cm.off('beforeChange', this.beforeChange);
      cm.off('cursorActivity', this.cursorActivity);
      cm.off('mousedown', this.clear);
    }
  }, {
    key: 'reset',
    value: function reset() {
      var _this = this;

      var cm = this.cm;
      this.clear();
      this.currentHint = '';
      this.dictionary.reset();
      var isWordChar = this.cm.getOption('source') === CSS ? isCSSWordChar : _TextUtils.TextUtils.isWordChar;
      this.isWordChar = cm.getOption('isWordChar') || isWordChar;

      this.textToWords(cm.getValue(), this.isWordChar, function (word) {
        if (word.length && (word[0] < '0' || word[0] > '9')) {
          _this.dictionary.addWord(word);
        }
      });

      this.keywords = keywords[cm.getOption('source')] || [];
      this.keywords.forEach(function (word) {
        return _this.dictionary.addWord(word);
      });

      this.userKeywords = cm.getOption('autocomplete-keywords') || [];
      this.userKeywords.forEach(function (word) {
        return _this.dictionary.addWord(word);
      });
    }
  }, {
    key: 'textToWords',
    value: function textToWords(text, isWordChar, wordCallback) {
      var startWord = -1;
      for (var i = 0; i < text.length; ++i) {
        if (!isWordChar(text.charAt(i))) {
          if (startWord !== -1) wordCallback(text.substring(startWord, i));
          startWord = -1;
        } else if (startWord === -1) {
          startWord = i;
        }
      }
      if (startWord !== -1) wordCallback(text.substring(startWord));
    }
  }, {
    key: 'getLines',
    value: function getLines(from, to) {
      return this.cm.getRange({ line: from.line, ch: 0 }, { line: to.line });
    }
  }, {
    key: 'dismiss',
    value: function dismiss() {
      this.clear();
    }
  }, {
    key: 'select',
    value: function select() {
      if (!this.currentHint) return;

      this.cm.replaceRange(this.currentHint, this.cursor, this.cursor, '+input');
      this.clear();
    }
  }, {
    key: 'showHint',
    value: function showHint(from, to) {
      this.clear();
      var cm = this.cm;

      // if at end of line, and have words to the left, then search the dictionary
      // and put the match in the hint and show it

      var cursor = cm.getCursor();
      var currentLine = cm.getLine(cursor.line);

      var word = '';
      var i = currentLine.length;
      while (i--) {
        var chr = currentLine[i];
        if (!this.isWordChar(chr)) {
          break;
        }
        word = chr + word;
      }

      if (!word) {
        return;
      }

      var matches = this.dictionary.wordsWithPrefix(word);
      console.log('word: %s, %s', word, matches);

      if (matches.length === 0) {
        return;
      }

      var match = matches[0].substr(word.length);

      this.hint.innerText = match;
      this.currentHint = match;

      this.mark = cm.setBookmark(cursor, {
        widget: this.hint,
        insertLeft: false
      });
    }
  }]);

  return Autocomplete;
}();

var _initialiseProps = function _initialiseProps() {
  var _this2 = this;

  this.keydown = function (cm, event) {
    if (!_this2.currentHint) {
      return;
    }

    switch (event.which) {
      case _Keys2.default.Tab.code:
      case _Keys2.default.Enter.code:
      case _Keys2.default.Right.code:
        _this2.select();
        event.preventDefault();
        break;
      default:
        _this2.dismiss();
    }
  };

  this.beforeChange = function (cm, change) {
    // clear the lines from change.from, and change.to
    var lines = _this2.getLines(change.from, change.to);
    _this2.textToWords(lines, _this2.isWordChar, function (word) {
      if (_this2.userKeywords.indexOf(word) === -1) {
        if (word.length && (word[0] < '0' || word[0] > '9')) {
          if (_this2.keywords.indexOf(word) === -1) {
            _this2.dictionary.removeWord(word);
          }
        }
      }
    });
  };

  this.cursorActivity = function () {
    var cursor = _this2.cm.getCursor();
    // dismiss the current hint

    if (cursor.line !== _this2.cursor.line && cursor.ch !== _this2.cursor.ch) {
      _this2.clear();
    }

    _this2.cursor = cursor;
  };

  this.change = function (cm, changes) {
    var from = changes.from,
        to = changes.to,
        origin = changes.origin;

    if (origin === 'setValue') {
      _this2.reset();
    }
    cm.startOperation();
    _this2.cursor = _this2.cm.getCursor();
    _this2.showHint(from, to);
    cm.endOperation();
  };

  this.clear = function () {
    if (_this2.mark) _this2.mark.clear();
    _this2.hint.innerText = '';
    _this2.currentHint = '';
  };
};